#PBS -l nodes=1:ppn=16:xeon16
#PBS -l walltime=2:15:0

import os
import sys
from glob import glob
from optparse import OptionParser
import numpy as np
from ase.visualize import view
from ase.io import read, write
from gpaw import GPAW, PoissonSolver, FermiDirac, Davidson, Mixer
from gpaw.poisson import FFTPoissonSolver
from gpaw.utilities import h2gpts
from gpaw.xc.libvdwxc import vdw_df, vdw_df2, vdw_df_cx
from gpaw.mpi import world
import gpaw.xc.gga as gga
from gpaw.fd_operators import Gradient


def get_gradient_ops(gd):
    return [Gradient(gd, v, n=2).apply for v in range(3)]
gga.get_gradient_ops = get_gradient_ops  # monkeypatch


p = OptionParser()
p.add_option('--s22', action='store_true')
p.add_option('--ref', action='store_true')
p.add_option('--all', action='store_true')

opts, args = p.parse_args()

calc_s22 = opts.s22 or opts.all
calc_ref = opts.ref or opts.all

cwd = os.getcwd()
path0, mydirname = os.path.split(cwd)
molno = int(mydirname.split('.')[1])
fnames = glob('../xyz_displaced/s%02d_??.xyz' % molno)
fnames.sort()

configs = [read(fname) for fname in fnames]

atoms = configs[-1]
atoms.pbc = 1
tmp = atoms.copy()
tmp.center(vacuum=10.0)
atoms.cell = tmp.cell # Atoms remain in corner (lower egg-box)

d1 = configs[1].positions[-1]
d2 = configs[2].positions[-1]
dx1 = d2 - d1
DX = configs[2].positions - configs[1].positions

i1 = [i for i, pos in enumerate(DX) if abs(np.vdot(dx1, DX[i])) > 1e-4]
i2 = [i for i, pos in enumerate(DX) if abs(np.vdot(dx1, DX[i])) < 1e-4]

mol1 = atoms[i1]
mol2 = atoms[i2]
assert len(mol1) + len(mol2) == len(atoms)

approx_optima = [None, 12, 9, 8, 8, 7, 7, 7, 18, 25, 20, 21, 18, 11, 18, 12,
                 12, 11, 15, 11, 17, 14, 10]

def getcalc(**kwargs):
    xc = vdw_df_cx()
    xc.get_setup_name = lambda: 'PBE'
    kwargs1 = dict(gpts=h2gpts(0.14, atoms.cell, idiv=16),
                   mode='fd',
                   eigensolver=Davidson(4),
                   basis='dzp',
                   convergence=dict(density=2e-5),
                   occupations=FermiDirac(0.0),
                   nbands='105%',
                   mixer=Mixer(0.25, 5, 50.0),
                   xc=xc,
                   poissonsolver=FFTPoissonSolver()
                   #poissonsolver=PoissonSolver(relax='GS', eps=2e-12)
                   )
    kwargs1.update(kwargs)
    calc = GPAW(**kwargs1)
    return calc


def lock_try(lockfname):
    decision = np.zeros(1, dtype=int)
    fd = None
    if world.rank == 0:
        if not os.path.exists(lockfname):
            decision[0] = 1
            fd = open(lockfname, 'w')
    world.sum(decision)
    decision = decision[0]
    if decision and world.rank == 0:
        print 'calculate %s' % lockfname
    return fd, decision


if calc_s22:
    atoms.calc = getcalc(txt='gpaw.s22.txt')
if calc_ref:
    mol1.calc = getcalc(txt='gpaw.mol1.txt')
    mol2.calc = getcalc(txt='gpaw.mol2.txt')    

for i, config in enumerate(configs):
    atoms.positions[:] = config.positions
    id = '%03d' % i

    if abs(i - approx_optima[molno]) > 3:
        print 'molno', molno, 'opt', approx_optima[molno], 'i', i, 'skip'
        continue

    lock_s22 = 'results.s22.%s.txt' % id
    lock_mol1 = 'results.mol1.%s.txt' % id
    lock_mol2 = 'results.mol2.txt'  # stationary

    mol1.positions[:] = atoms.positions[i1]
    mol2.positions[:] = atoms.positions[i2]

    x = np.linalg.norm(mol2.get_center_of_mass() - mol1.get_center_of_mass())

    if calc_s22:
        fd, decision = lock_try(lock_s22)
        if decision:
            atoms.calc = getcalc(txt='gpaw.s22.%s.txt' % id)
            E = atoms.get_potential_energy()
            F0 = atoms.get_forces()
            write('out.s22.%s.traj' % id, atoms)
            if fd is not None:
                fd.write('%15.10f %15.10f\n' % (x, E))
                fd.close()

    if calc_ref:
        fd, decision = lock_try(lock_mol2)
        if decision:
            Eref2 = mol2.get_potential_energy()
            Fref2 = mol2.get_forces()
            write('out.mol2.traj', mol2)
            if fd is not None:
                fd.write('%15.10f %15.10f\n' % (x, Eref2))
                fd.close()
            mol2.calc = None


    if calc_ref:
        fd, decision = lock_try(lock_mol1)
        if decision:
            mol1.calc = getcalc(txt='gpaw.mol1.%s.txt' % id)
            Eref1 = mol1.get_potential_energy()
            Fref1 = mol1.get_forces()
            write('out.mol1.%s.traj' % id, mol1)
            if fd is not None:
                fd.write('%15.10f %15.10f\n' % (x, Eref1))
                fd.close()
